;Colour bars program
;-------------------
;Binary created using DAsm 2.12 running on an Amiga.

   PROCESSOR   6502

BALLPHIY    EQU   #$0000
BALLPLOY    EQU   #$0001
BALLPHIX    EQU   #$0002
BALLPLOX    EQU   #$0003
BALLVLOY    EQU   #$0004
BALLVHIY    EQU   #$0005
BALLVLOX    EQU   #$0006
BALLVHIX    EQU   #$0007
BALLACCY    EQU   #$0008
BALLACCX    EQU   #$0009
BALLVISIBLE EQU   #$000A

   ORG   $C000    ;16Kb PRG-ROM, 8Kb CHR-ROM

Reset_Routine  SUBROUTINE
   cld         ;Clear decimal flag
   sei         ;Disable interrupts
.WaitV   lda $2002
   bpl .WaitV     ;Wait for vertical blanking interval
   ldx #$00
   stx $2000
   stx $2001      ;Screen display off, amongst other things
   dex
   txs         ;Top of stack at $1FF

;Clear (most of) the NES' WRAM. This routine is ripped from "Duck Hunt" - I should probably clear all
;$800 bytes.
   ldy #$06    ;To clear 7 x $100 bytes, from $000 to $6FF?
   sty $01        ;Store count value in $01
   ldy #$00
   sty $00
   lda #$00

.Clear   sta ($00),y    ;Clear $100 bytes
   dey
   bne .Clear

   dec $01        ;Decrement "banks" left counter
   bpl .Clear     ;Do next if >= 0


   lda   #$20
   sta   $2006
   lda   #$00
   sta   $2006

   ldx   #$00
   ldy   #$10
.ClearPPU sta $2007
   dex
   bne   .ClearPPU
   dey
   bne   .ClearPPU

;********* Initialize Palette to specified colour ********

   ldx   #$3F
   stx   $2006
   ldx   #$00
   stx   $2006

   ldx   #$0D     ;Colour Value (Black)
   ldy   #$20     ;Clear BG & Sprite palettes.
.InitPal stx $2007
   dey
   bne   .InitPal
;*********************************************************

   lda   #$3F
   sta   $2006
   lda   #$11
   sta   $2006
   lda   #$30
   sta   $2007
   sta   $2007
   sta   $2007

   lda   #$00           ;Set up sprite palette & position
   sta   $2003

   lda   #$00
   sta   BALLPHIY
   lda   #$E0
   sta   BALLPLOY

   lda   #$00
   sta   BALLPHIX
   lda   #$20
   sta   BALLPLOX

   lda   #$02
   sta   BALLVHIY
   lda   #$00
   sta   BALLVLOY

   lda   #$00
   sta   BALLVHIX
   lda   #$40
   sta   BALLVLOX

   lda   #$05
   sta   BALLACCY
   lda   #$00
   sta   BALLACCX

   lda   #$02
   sta   BALLVISIBLE

   lda   BALLPLOY
   sta   $2004
   lda   BALLVISIBLE
   sta   $2004
   lda   #$00
   sta   $2004
   lda   BALLPLOX
   sta   $2004

;Enable vblank interrupts, etc.
   lda   #%10001000
   sta   $2000
   lda   #%00011110        ;Screen on, sprites on, show leftmost 8 pixels, colour
   sta   $2001
;  cli            ;Enable interrupts(?)

;Now just loop forever?
.Loop jmp   .Loop

Gravity SUBROUTINE
   lda   BALLVLOY
   sec
   sbc   BALLACCY
   sta   BALLVLOY
   bcs   .NoOverflowVY
   dec   BALLVHIY
.NoOverflowVY
   lda   BALLVHIY
   pha
   lda   BALLVLOY
   pha
   lsr   BALLVHIY
   ror   BALLVLOY
   lsr   BALLVHIY
   ror   BALLVLOY
   lsr   BALLVHIY
   ror   BALLVLOY
   lsr   BALLVHIY
   ror   BALLVLOY
   lsr   BALLVHIY
   ror   BALLVLOY
   lsr   BALLVHIY
   ror   BALLVLOY

   lda   BALLPLOY
   sec
   sbc   BALLVLOY
   sta   BALLPLOY

   pla
   sta   BALLVLOY
   pla
   sta   BALLVHIY

;---------------------- X-POS -------------
   lda   BALLVLOX
   clc
   adc   BALLACCX
   sta   BALLVLOX
   bcc   .NoOverflowX
   lda   BALLVHIX
   sec
   adc   #$00
   sta   BALLVHIX
.NoOverflowX
   lda   BALLVHIX
   pha
   lda   BALLVLOX
   pha
   lsr   BALLVHIX
   ror   BALLVLOX
   lsr   BALLVHIX
   ror   BALLVLOX
   lsr   BALLVHIX
   ror   BALLVLOX
   lsr   BALLVHIX
   ror   BALLVLOX
   lsr   BALLVHIX
   ror   BALLVLOX
   lsr   BALLVHIX
   ror   BALLVLOX

   lda   BALLPLOX
   clc
   adc   BALLVLOX
   sta   BALLPLOX

   pla
   sta   BALLVLOX
   pla
   sta   BALLVHIX

;------------ TESTING FOR OFF-SCREEN ------------------------------
   lda   BALLPHIY
   beq   .Test1
   dec   BALLPHIY
   beq   .Test2
   dec   BALLPHIY
   beq   .Test3

.Test1 lda   #$10                    ;Disappear if goes too high
   sec
   sbc   BALLPLOY
   bcc   .GravEnd
                                     ;HOW DOES THIS WORK????????
   lda   #$02
   sta   BALLVISIBLE
   lda   #$01
   sta   BALLPHIY
   jmp   .GravEnd

.Test2
   lda   #$01
   sta   BALLPHIY
   lda   #$F0
   sec
   sbc   BALLPLOY
   bcs   .GravEnd

   lda   #$01
   sta   BALLVISIBLE
   lda   #$02
   sta   BALLPHIY
   jmp   .GravEnd

.Test3
   lda   #$02
   sta   BALLPHIY
   lda   #$40
   sec
   sbc   BALLPLOY
   bcs   .GravEnd

   lda   #$00
   sta   BALLPHIY


.GravEnd
   rts

NMI_Routine SUBROUTINE
   php
   pha
   txa
   pha
   tya
   pha

   jsr   Gravity

   lda   #$00
   sta   $2003
   lda   BALLPLOY
   sta   $2004
   lda   BALLVISIBLE
   sta   $2004
   lda   #$00
   sta   $2004
   lda   BALLPLOX
   sta   $2004

   pla
   tay
   pla
   tax
   pla
   plp
   rti

IRQ_Routine       ;Dummy label
   rti

;That's all the code. Now we just need to set the vector table approriately.

   ORG   $FFFA,0
   dc.w  NMI_Routine
   dc.w  Reset_Routine
   dc.w  IRQ_Routine    ;Not used, just points to RTI


;The end.
